import getopt, sys,math,time
from nifti import NiftiImage
from opencv.cv import * #@UnusedWildImport
from opencv.highgui import * #@UnusedWildImport
import scipy as sc



def usage():
    print(sys.argv[0] + " --verbose --help --output out.nii.gz in.nii.gz")

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "ho:v", ["help", "output="])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return 2
        
    if(len(args) != 1):
        usage()
        return 2
    else:
        input = args[0]
        
    output = None
    verbose = False
    for o, a in opts:
        if o == "-v":
            verbose = True
        elif o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-o", "--output"):
            output = a
        else:
            assert False, "unhandled option"
            
    cvimg = nifti2opencv(input, False)
    orig_img = cvCloneImage(cvimg)
    segmentation(cvimg, True)
    threshold(cvimg)
    cvimg, contour = contours(cvimg)
    curvature = getCurvature(contour, 5)
    ends = findEnds(curvature, contour, cvGetSize(cvimg))
    findAxisCenter(contour, ends)
    dist_map = distanceMap(cvimg, True)
    findAxisSnake(contour, ends, dist_map)
    #distanceMap(cvimg, True)
    cvReleaseImage(cvimg)
    cvReleaseImage(orig_img)
    
def slice2opencv(slice, depth):
    cvimg = cvCreateImage(cvSize(slice.shape[1], slice.shape[0]), depth, 1)
    for x in range(len(slice)):
        for y in range(len(slice[x])):
            cvSet2D(cvimg, x, y, cvScalar(float(slice[x][y])))
    return cvimg

def nifti2opencv(nifti_filename, verbose=False):
    middle_slice = 127
    nim3d = NiftiImage(nifti_filename)
    if nim3d.header['datatype'] == 1:
        #binary
        depth = IPL_DEPTH_8U
    elif nim3d.header['datatype'] == 2:
        depth = IPL_DEPTH_8U
    elif nim3d.header['datatype'] == 4:
        depth = IPL_DEPTH_16S
    elif nim3d.header['datatype'] == 8:
        depth = IPL_DEPTH_32S
    elif nim3d.header['datatype'] == 16:
        depth = IPL_DEPTH_32F
    elif nim3d.header['datatype'] == 64:
        depth = IPL_DEPTH_64F
    else:
        assert False, "unknown datatype: " + nim3d.header['datatype']
     
    slice = nim3d.data[::-1, :, middle_slice]
    
    cvimg = slice2opencv(slice, depth)
        
    if verbose:     
        showImg("original", cvimg)
        
    return cvimg
    
def segmentation(cvimg, verbose = False):
    seedPoint = cvPoint(128, 99)
    cvFloodFill(cvimg, seedPoint, cvScalar(255), cvScalarAll(10), cvScalarAll(11))
    if verbose:        
        showImg("segmented", cvimg)
    return cvimg
        
def threshold(cvimg, verbose = False):
    cvThreshold(cvimg, cvimg, 254, 255, CV_THRESH_BINARY)
    if verbose:        
        showImg("threshold", cvimg)
    return cvimg
def contours(cvimg, verbose = False):
    storage = cvCreateMemStorage(0)
    num_contours, contours = cvFindContours(cvimg, storage, sizeof_CvContour, CV_RETR_LIST, CV_CHAIN_APPROX_NONE) #@UnusedVariable
    contour_list = list()
    for contour in contours.hrange():
        for pt in contour:
            contour_list.append((pt.x, pt.y))
    cvZero(cvimg)
    cvDrawContours(cvimg, contours, cvScalar(255), cvScalar(255), 3)
    cvNot(cvimg, cvimg)
    if verbose:        
        showImg("contours", cvimg)
    cvReleaseMemStorage(storage)
            
    return (cvimg, contour_list)

def distanceMap(cvimg, verbose = False):
    dst = cvCreateImage(cvGetSize(cvimg), IPL_DEPTH_32F, 1)
    bin = cvCreateImage(cvGetSize(cvimg), IPL_DEPTH_8U, 1)
    cvThreshold(cvimg, bin, 1, 1, CV_THRESH_BINARY)
    cvDistTransform(bin, dst)
    if verbose:   
        scaled = cvClone(dst)     
        cvScale(scaled, scaled, 0.15)
        showImg("distance map", scaled)
    return dst

def getCurvature(contour, k, cvimg=None):
    curvature_list = list()
    for i in range(len(contour)):
        x = contour[i][0]
        x_pred = contour[i-k][0]
        x_succ = contour[(i+k) % len(contour)][0]
        a_2, a_1, a_0 = sc.polyfit([1, 0, 2], [x_pred, x, x_succ], 2) #@UnusedVariable
        
        y = contour[i][1]
        y_pred = contour[i-k][1]
        y_succ = contour[(i+k) % len(contour)][1]
        b_2, b_1, b_0 = sc.polyfit([1, 0, 2], [y_pred, y, y_succ], 2) #@UnusedVariable
        
        curv = 2*(a_1*b_2 - b_1*a_2)/math.pow(a_1**2 +b_1**2, 1.5)
        #print "x = %d, y = %d, curvature = %f" % (x,y,curv)

        curvature_list.append(curv)
    
    if cvimg != None:
        cvZero(cvimg)
        for i in range(len(curvature_list)):
            curv = curvature_list[i]
            x = contour[i][0]
            y = contour[i][1]
            curv_norm = (curv - min(curvature_list))/(max(curvature_list)-min(curvature_list))*255
            cvSet2D(cvimg, y, x, cvScalar(curv_norm))
        showImg("curvature", cvimg)
    return curvature_list

def findEnds(curvature, contour, img_size, cvimg=None):
    max_left = 0
    max_left_coo = None
    max_right = 0
    max_right_coo = None
    for i in range(len(curvature)):
        x = contour[i][0]
        y = contour[i][1]
        if x < img_size.width/2:
            if max_left < curvature[i]:
                max_left = curvature[i]
                max_left_coo = (x, y)
        else:
            if max_right < curvature[i]:
                max_right = curvature[i]
                max_right_coo = (x, y)
    if cvimg != None:
        cvSet2D(cvimg, max_left_coo[1], max_left_coo[0], cvScalar(255))
        cvSet2D(cvimg, max_right_coo[1], max_right_coo[0], cvScalar(255))
        showImg("ends", cvimg)
    
    return (max_left_coo, max_right_coo)

def findAxisCenter(contour, ends, cvimg=None):
    left_end_index = contour.index(ends[0])
    right_end_index = contour.index(ends[1])
    lower_contour = list()
    upper_contour = list()
    # TODO
    if left_end_index < right_end_index:
        lower_contour = contour[left_end_index+1:right_end_index-1]
        upper_contour = contour[right_end_index+1:] + contour[:left_end_index-1]
    else:
        lower_contour = contour[left_end_index+1:] + contour[:right_end_index-1]
        upper_contour = contour[right_end_index+1:left_end_index-1]
        
    last_index = 0
    axis = list()
    axis.append(ends[1])
    for i in range(len(upper_contour)):
        min_dist = 1000000
        min_dist_vec = 0
        index = 0

        for j in range(last_index-1,len(lower_contour)):
            vec = (upper_contour[i][0]-lower_contour[j][0],  upper_contour[i][1]-lower_contour[j][1])
            dist = math.sqrt(vec[0]**2 + vec[1]**2)
            if(dist < min_dist):
                min_dist = dist
                min_dist_vec = vec
                index = j
        last_index = index
        axis_point = (upper_contour[i][0]-min_dist_vec[0]/2.0, upper_contour[i][1]-min_dist_vec[1]/2.0)
        axis.append(axis_point)
    axis.append(ends[0])
    print lower_contour
    print upper_contour
    print axis
    if cvimg != None:
        for i in range(len(axis)):
            cvSet2D(cvimg, int(round(axis[i][1],0)), int(round(axis[i][0],0)), cvScalar(0))
            showImg("axis", cvimg, 0.5)
        for i in range(len(lower_contour)):
            cvSet2D(cvimg, int(lower_contour[i][1]), int(lower_contour[i][0]), cvScalar(100))
        for i in range(len(upper_contour)):
            cvSet2D(cvimg, int(upper_contour[i][1]), int(upper_contour[i][0]), cvScalar(250))
        showImg("axis", cvimg)

def findAxisSnake(contour, ends, dist_map):
    contour_points = list()
    for pt in contour:
        contour_points.append(cvPoint(pt[0],pt[1]))
    x = float(1.0)
    #TODO
    #cvSnakeImage(dist_map, contour_points[0], len(contour_points), x,1,1, CV_VALUE, cvSize(3,3), cvTermCriteria(CV_TERMCRIT_ITER, 100,0.0))
    print contour_points
        
def showImg(label, image, period = 0):
    cvStartWindowThread()
    cvNamedWindow(label)
    cvShowImage(label, image)
    if period != 0:
        time.sleep(period)
    else:
        sys.stdin.read(1)

if __name__ == '__main__':
    sys.exit(main())
